package java.nio;

import sun.misc.Cleaner;
import sun.misc.Unsafe;
import sun.nio.ch.DirectBuffer;

class DirectDoubleBufferU extends DoubleBuffer implements DirectBuffer
{
    protected static final Unsafe unsafe;
    private static final long arrayBaseOffset;
    protected static final boolean unaligned;
    private final Object att;
    
    @Override
    public Object attachment() {
        return this.att;
    }
    
    public Cleaner cleaner() {
        return null;
    }
    
    DirectDoubleBufferU(DirectBuffer att, int n, int n2, int n3, int n4, int n5) {
        super(n, n2, n3, n4);
        this.address = att.address() + (long)n5;
        this.att = att;
    }
    
    @Override
    public DoubleBuffer slice() {
        int position = this.position();
        int limit = this.limit();
        int n = (position <= limit) ? (limit - position) : 0;
        int n2 = position << 3;
        assert n2 >= 0;
        return new DirectDoubleBufferU(this, -1, 0, n, n, n2);
    }
    
    @Override
    public DoubleBuffer duplicate() {
        return new DirectDoubleBufferU(this, this.markValue(), this.position(), this.limit(), this.capacity(), 0);
    }
    
    @Override
    public DoubleBuffer asReadOnlyBuffer() {
        return new DirectDoubleBufferRU(this, this.markValue(), this.position(), this.limit(), this.capacity(), 0);
    }
    
    @Override
    public long address() {
        return this.address;
    }
    
    private long ix(int n) {
        return this.address + ((long)n << 3);
    }
    
    @Override
    public double get() {
        return DirectDoubleBufferU.unsafe.getDouble(this.ix(this.nextGetIndex()));
    }
    
    @Override
    public double get(int n) {
        return DirectDoubleBufferU.unsafe.getDouble(this.ix(this.checkIndex(n)));
    }
    
    @Override
    public DoubleBuffer get(double[] array, int n, int n2) {
        if ((long)n2 << 3 > 6L) {
            Buffer.checkBounds(n, n2, array.length);
            int position = this.position();
            int limit = this.limit();
            assert position <= limit;
            if (n2 > ((position <= limit) ? (limit - position) : 0)) {
                throw new BufferUnderflowException();
            }
            if (this.order() != ByteOrder.nativeOrder()) {
                Bits.copyToLongArray(this.ix(position), array, (long)n << 3, (long)n2 << 3);
            }
            else {
                Bits.copyToArray(this.ix(position), array, DirectDoubleBufferU.arrayBaseOffset, (long)n << 3, (long)n2 << 3);
            }
            this.position(position + n2);
        }
        else {
            super.get(array, n, n2);
        }
        return this;
    }
    
    @Override
    public DoubleBuffer put(double x) {
        DirectDoubleBufferU.unsafe.putDouble(this.ix(this.nextPutIndex()), x);
        return this;
    }
    
    @Override
    public DoubleBuffer put(int n, double x) {
        DirectDoubleBufferU.unsafe.putDouble(this.ix(this.checkIndex(n)), x);
        return this;
    }
    
    @Override
    public DoubleBuffer put(DoubleBuffer doubleBuffer) {
        if ((doubleBuffer instanceof DirectDoubleBufferU)) {
            if (doubleBuffer == this) {
                throw new IllegalArgumentException();
            }
            DirectDoubleBufferU directDoubleBufferU = (DirectDoubleBufferU)doubleBuffer;
            int position2 = directDoubleBufferU.position();
            int limit2 = directDoubleBufferU.limit();
            assert position2 <= limit2;
            int n2 = (position2 <= limit2) ? (limit2 - position2) : 0;
            int position3 = this.position();
            int limit3 = this.limit();
            assert position3 <= limit3;
            if (n2 > ((position3 <= limit3) ? (limit3 - position3) : 0)) {
                throw new BufferOverflowException();
            }
            DirectDoubleBufferU.unsafe.copyMemory(directDoubleBufferU.ix(position2), this.ix(position3), (long)n2 << 3);
            directDoubleBufferU.position(position2 + n2);
            this.position(position3 + n2);
        }
        else if (doubleBuffer.hb != null) {
            int position = doubleBuffer.position();
            int limit = doubleBuffer.limit();
            assert position <= limit;
            int n = (position <= limit) ? (limit - position) : 0;
            this.put(doubleBuffer.hb, doubleBuffer.offset + position, n);
            doubleBuffer.position(position + n);
        }
        else {
            super.put(doubleBuffer);
        }
        return this;
    }
    
    @Override
    public DoubleBuffer put(double[] array, int n, int n2) {
        if ((long)n2 << 3 > 6L) {
            Buffer.checkBounds(n, n2, array.length);
            int position = this.position();
            int limit = this.limit();
            assert position <= limit;
            if (n2 > ((position <= limit) ? (limit - position) : 0)) {
                throw new BufferOverflowException();
            }
            if (this.order() != ByteOrder.nativeOrder()) {
                Bits.copyFromLongArray(array, (long)n << 3, this.ix(position), (long)n2 << 3);
            }
            else {
                Bits.copyFromArray(array, DirectDoubleBufferU.arrayBaseOffset, (long)n << 3, this.ix(position), (long)n2 << 3);
            }
            this.position(position + n2);
        }
        else {
            super.put(array, n, n2);
        }
        return this;
    }
    
    @Override
    public DoubleBuffer compact() {
        int position = this.position();
        int limit = this.limit();
        assert position <= limit;
        int n = (position <= limit) ? (limit - position) : 0;
        DirectDoubleBufferU.unsafe.copyMemory(this.ix(position), this.ix(0), (long)n << 3);
        this.position(n);
        this.limit(this.capacity());
        this.discardMark();
        return this;
    }
    
    @Override
    public boolean isDirect() {
        return true;
    }
    
    @Override
    public boolean isReadOnly() {
        return false;
    }
    
    @Override
    public ByteOrder order() {
        return (ByteOrder.nativeOrder() != ByteOrder.BIG_ENDIAN) ? ByteOrder.LITTLE_ENDIAN : ByteOrder.BIG_ENDIAN;
    }
    
    static {
        unsafe = Bits.unsafe();
        arrayBaseOffset = (long)DirectDoubleBufferU.unsafe.arrayBaseOffset(double[].class);
        unaligned = Bits.unaligned();
    }
}
